/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.action.interaction.selenium;

import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultElementLocator;
import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultOptionLocator;
import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultWindowLocator;

import com.hamburgsud.aludratest.action.Action;
import com.hamburgsud.aludratest.action.interaction.Interaction;
import com.hamburgsud.aludratest.caller.Command;
import com.hamburgsud.aludratest.caller.TearDownCommand;
import com.hamburgsud.aludratest.handler.DefaultInteractionExceptionHandler;
import com.hamburgsud.aludratest.handler.ExceptionHandler;
import com.hamburgsud.aludratest.locator.element.ElementLocator;
import com.hamburgsud.aludratest.locator.option.OptionLocator;
import com.hamburgsud.aludratest.locator.window.WindowLocator;
import com.hamburgsud.aludratest.wrapper.Wrapper;
import com.hamburgsud.log4testing.core.data.TestCase;
import com.hamburgsud.log4testing.core.data.TestStep;

public class SeleniumInteraction extends Action implements Interaction {

	public SeleniumInteraction(final Wrapper wrapper, final TestCase testCase) {
		super(wrapper, testCase);
	}

	@Override
	public ExceptionHandler getExceptionHandler() {
		if (exceptionHandler == null) {
			exceptionHandler = new DefaultInteractionExceptionHandler();
		}
		return super.getExceptionHandler();
	}

	public TestStep changeCheckbox(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			public void call() throws Exception {
				wrapper.click(elementLocator);
			}

			public String getName() {
				return "changeCheckbox";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep deselectCheckbox(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				if (wrapper.isChecked(elementLocator)) {
					wrapper.click(elementLocator);
				}
			}

			@Override
			public String getName() {
				return "deselectCheckbox";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public String getInputFieldValue(final Object locator) {
		final StringBuilder inputFieldValue = new StringBuilder(0);
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		caller.call(new Command() {

			@Override
			public void call() throws Exception {
				final String readValue = wrapper.getValue(elementLocator);
				if (readValue != null) {
					inputFieldValue.append(readValue);
				}
			}

			@Override
			public String getName() {
				return "getInputFieldValue";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(
						getArgumentMessage(inputFieldValue.toString()),
						getLocatorMessage(elementLocator));
			}

		});
		return inputFieldValue.toString();
	}

	public TestStep selectCheckbox(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				if (!wrapper.isChecked(elementLocator)) {
					wrapper.click(elementLocator);
				}
			}

			@Override
			public String getName() {
				return "selectCheckbox";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep selectDropDownEntry(final Object locator,
			final Object optionLocator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		final OptionLocator optLocator = getDefaultOptionLocator(wrapper,
				optionLocator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.select(elementLocator, optLocator);
			}

			@Override
			public String getName() {
				return "selectDropDownEntry";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator, optLocator);
			}

		});
	}

	public TestStep selectRadiobutton(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.click(elementLocator);
			}

			@Override
			public String getName() {
				return "selectRadiobutton";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep type(final Object locator, final String text) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.type(elementLocator, text);
			}

			@Override
			public String getName() {
				return "type";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(text), getLocatorMessage(elementLocator));
			}

		});
	}

	public TestStep open(final String url) {
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.open(url);
			}

			@Override
			public String getName() {
				return "open";
			}

			@Override
			public String getMessage() {
				return url;
			}

		});
	}

	public TestStep refresh() {
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.refresh();
			}

			@Override
			public String getName() {
				return "refresh";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep tearDown() {
		return caller.call(new TearDownCommand() {

			@Override
			public void call() throws Exception {
				wrapper.tearDown();
			}

			@Override
			public String getName() {
				return "tearDown";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep waitForPageToLoad() {
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.waitForPageToLoad();
			}

			@Override
			public String getName() {
				return "waitForPageToLoad";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}
	
	public TestStep waitForEnabled(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.waitForEnabled(elementLocator);
			}

			@Override
			public String getName() {
				return "waitForEnabled";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep windowFocus() {
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.windowFocus();
			}

			@Override
			public String getName() {
				return "windowFocus";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep windowMaximize() {
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.windowMaximize();
			}

			@Override
			public String getName() {
				return "windowMaximize";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep selectWindow(final Object locator) {
		final WindowLocator windowLocator = getDefaultWindowLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.selectWindow(windowLocator);
			}

			@Override
			public String getName() {
				return "selectWindow";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(windowLocator);
			}

		});
	}

	public TestStep focus(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.focus(elementLocator);
			}

			@Override
			public String getMessage() {
				return elementLocator.toString();
			}

			@Override
			public String getName() {
				return "focus";
			}
		});
	}

	public TestStep keyPress(final int keycode) {
		return caller.call(new Command() {

			@Override
			public String getName() {
				return "keyPress";
			}

			@Override
			public String getMessage() {
				return String.valueOf(keycode);
			}

			@Override
			public void call() throws Exception {
				wrapper.keyPress(keycode);
			}
		});
	}

	public TestStep doubleClick(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.doubleClick(elementLocator);
			}

			@Override
			public String getName() {
				return "doubleClick";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep click(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.click(elementLocator);
			}

			@Override
			public String getName() {
				return "click";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep closeOtherWindows(final Object locator) {
		final WindowLocator stillOpenedWindow = getDefaultWindowLocator(
				wrapper, locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				final String[] windowTitles = wrapper.getAllWindowTitles();
				for (String windowTitle : windowTitles) {
					final WindowLocator windowToClose = getDefaultWindowLocator(
							wrapper, windowTitle);
					if (!windowToClose.equals(stillOpenedWindow)) {
						wrapper.selectWindow(windowToClose);
						wrapper.close();
					}
				}
				wrapper.selectWindow(stillOpenedWindow);
			}

			@Override
			public String getName() {
				return "closeOtherWindows";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(stillOpenedWindow);
			}
		});

	}

	public TestStep waitForInvisible(Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,locator);
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.waitForInvisible(elementLocator);
			}

			@Override
			public String getName() {
				return "waitForInvisible";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep waitForCondition(final String script) {
		return caller.call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.waitForCondition(script);
			}

			@Override
			public String getName() {
				return "waitForCondition";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public int getXPathCount(String xpath) {
		return wrapper.getXPathCount(xpath).intValue();
	}

}
