/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.action.verification;

import java.util.Set;

import com.hamburgsud.aludratest.action.interaction.Interaction;
import com.hamburgsud.aludratest.gui.component.Table.Row;
import com.hamburgsud.aludratest.locator.LocatorProvider;
import com.hamburgsud.log4testing.core.data.TestStatus;
import com.hamburgsud.log4testing.core.data.TestStep;

/**
 * The interface {@link Verification} provides several methods to verify values
 * in the active screen of the application under test. Every class which
 * implements this interface must assure that a call of one of these methods
 * verifies the expected value with the current in the application under test.<br/>
 * Methods with locators accept two types of locators.
 * <ol>
 * <li>The first type is a by the caller typed locator.</li>
 * <li>The second type is just a String which will be automatically transformed
 * into the default locator. In which locator the String will be transformed
 * depends on the implementation of each method of this interface. Normally
 * these methods call
 * {@link LocatorProvider#getDefaultElementLocator(net.sourceforge.aludratest.wrapper.Wrapper, String)}
 * to get the default locator for the corresponding wrapper.</li>
 * </ol>
 * For interactions with the application under test see {@link Interaction}.
 */
public interface Verification {

	/**
	 * Verifies that <code>label</code> is selected in the by
	 * <code>locator</code> determined dropdownlist.
	 * 
	 * @param locator
	 *            to locate one specific dropdownlist in the application under
	 *            test
	 * @param label
	 *            is the label of the entry which must be selected
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep dropDownEntrySelection(final Object locator,
			final String label);

	/**
	 * Verifies that one specific element, normally a label,
	 * has the expected text.
	 * 
	 * @param locator
	 *            to locate one specific label in the application under
	 *            test.
	 * @param expectedText
	 *            is the text the element must has
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep text(final Object locator, final String expectedText);

	/**
	 * Verifies that one specific element, normally a label,
	 * has the expected number.
	 * 
	 * @param locator
	 *            to locate one specific label in the application under
	 *            test.
	 * @param expectedNumber
	 *            is the number the element must has
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep text(final Object locator, final Number expectedNumber);

	/**
	 * Verifies the visibility of several elements. Invisible elements should be
	 * reported in the returned <code>TestStep</code> object.
	 * 
	 * @param locators
	 *            of the elements which shall be verified for their visibility
	 * @return a logging object which contains a report about invisible
	 *         elements. This object is normally just for the purpose of logging
	 */
	public TestStep visibility(final Object... locators);

	/**
	 * Verifies for several elements that they are editable. Uneditable elements
	 * should be reported in the returned <code>TestStep</code> object.
	 * 
	 * @param locators
	 *            of the elements which shall be verified for their editability
	 * @return a logging object which contains a report about uneditable
	 *         elements. This object is normally just for the purpose of logging
	 */
	public TestStep isEditable(final Object... locators);

	/**
	 * Verifies for several elements that they are <b>not</b> editable. Editable
	 * elements should be reported in the returned <code>TestStep</code> object.
	 * 
	 * @param locators
	 *            of the elements which shall be verified for their none
	 *            editability
	 * @return a logging object which contains a report about editable elements.
	 *         This object is normally just for the purpose of logging
	 */
	public TestStep isNotEditable(final Object... locators);

	/**
	 * Verifies that one specific element is present.
	 * 
	 * @param locator
	 *            to locate one specific element in the application under test
	 * @return a logging object which gives some hints if the element could not
	 *         be found. If the element is present, this object will have the
	 *         status {@link TestStatus#OK}.
	 */
	public TestStep isElementPresent(final Object locator);

	/**
	 * Verifies that elements are checked. This method will be often used for
	 * checkboxes and radiobuttons.
	 * 
	 * @param locators
	 *            of the elements which shall be verified that they are checked
	 * @return a logging object which contains a report about the unchecked
	 *         elements. This object is normally just for the purpose of logging
	 */
	public TestStep isChecked(final Object... locators);

	/**
	 * Verifies that the element has the focus.
	 * 
	 * @param locator
	 *            of the element which must have the focus
	 * @return a logging object which contains a report about the unchecked
	 *         elements. This object is normally just for the purpose of logging
	 */
	public TestStep hasFocus(final Object locator);

	/**
	 * Verifies that the located element has the passed through values.
	 * 
	 * @param locator
	 *            of the element which must contain the passed through values
	 * @param expectedValues
	 *            which must be contained by the located element
	 * @return a logging object which has the information if the located element
	 *         has the values or not. If not the logging object must provide the
	 *         information which values are not in the located element.
	 */
	public TestStep hasValues(final Object locator,
			final String[] expectedValues);

	/**
	 * Verifies that the located element has the passed trough labels.
	 * 
	 * @param locator
	 *            of the element which must contain the passed through labels
	 * @param expectedLabels
	 *            which must be contained by the located element
	 * @return a logging object which has the if the located element has the
	 *         labels or not. If not the logging object must provide the
	 *         information which values are not in the located element.
	 */
	public TestStep hasLabels(final Object locator,
			final String[] expectedLabels);

	/**
	 * Verifies that an input field has the expected length.
	 * 
	 * @param locator
	 *            which identifies an element
	 * @param attributeName
	 *            is the name of the attribute whose value shall be compared
	 *            with the expected value. Be aware that the identified element
	 *            has this attribute
	 * @param expectedValue
	 *            is the property the input field must have
	 * @return an logging object which provides the information if the check was
	 *         ok or not. If the check was not ok, then it shall provide some
	 *         information about the actual value of the attribute.
	 */
	public TestStep hasAttributeValue(final Object locator,
			final String attributeName, final String expectedValue);

	public TestStep getTable(final Object locator, int row, int col, String expected);

	public TestStep getXPathCount(final String xpath, final int expected);

	/**
	 * Verifies the element's checked status (checked or unchecked). This 
	 * method will be often used for checkboxes and radiobuttons.
	 * 
	 * @param expected 
	 *        true=checked, false=unchecked
	 * @param locator
	 *            of the element which shall be verified that they are checked
	 *            or unchecked
	 * @return a logging object which contains a report about the unchecked
	 *         elements that are expected to be checked and vice versa. 
	 *         This object is normally just for the purpose of logging
	 */
	public TestStep checked(final boolean expected, final Object locator);

	/**
	 * Verifies that one specific element, normally an input field or text area,
	 * has the expected text.
	 * 
	 * @param locator
	 *            to locate one specific element in the application under
	 *            test.
	 * @param expectedText
	 *            is the text the element must has
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep value(final Object locator, final String expectedText);

	/**
	 * Verifies that one specific element, normally an input field or text area,
	 * has the expected number.
	 * 
	 * @param locator
	 *            to locate one specific element in the application under
	 *            test.
	 * @param expectedNumber
	 *            is the number the element must has
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep value(final Object locator, final Number expectedNumber);

	/**
	 * Verifies that the located table has the passed through rows.
	 * 
	 * @param locator
	 *            of the table which must contain the passed through rows
	 * @param rows
	 *            which must be contained by the located table
	 * @return a logging object which has the information if the located table
	 *         has the rows or not.
	 */
	public TestStep tableRows(final Object locator, final Set<Row> rows);

}