/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.wrapper.selenium;

import java.util.ArrayList;

import com.hamburgsud.aludratest.configuration.SeleniumWrapperConfiguration;
import com.hamburgsud.aludratest.exception.technical.LocatorNotSupportedException;
import com.hamburgsud.aludratest.locator.Locator;
import com.hamburgsud.aludratest.locator.element.CSSLocator;
import com.hamburgsud.aludratest.locator.element.DOMLocator;
import com.hamburgsud.aludratest.locator.element.ElementLocator;
import com.hamburgsud.aludratest.locator.element.ElementLocators;
import com.hamburgsud.aludratest.locator.element.IdLocator;
import com.hamburgsud.aludratest.locator.element.LabelLocator;
import com.hamburgsud.aludratest.locator.element.XPathLocator;
import com.hamburgsud.aludratest.locator.option.OptionLocator;
import com.hamburgsud.aludratest.locator.window.TitleLocator;
import com.hamburgsud.aludratest.locator.window.WindowLocator;
import com.thoughtworks.selenium.CommandProcessor;
import com.thoughtworks.selenium.DefaultSelenium;
import com.thoughtworks.selenium.Selenium;

public class SeleniumFacade {

	// TODO The following scripts should be outsourced in selenium user
	// extension
	// javascripts which itself should be publicly made available
	final private static String HAS_FOCUS_SCRIPT = "this.browserbot.findElement('%s') == window.document.activeElement";
	final private static String DELIMITER = ";;;";
	final private static String DROPDOWN_SCRIPT = "var dropdownbox = this.browserbot.findElement('%s');"
			+ "var values = '';"
			+ "for (var i = 0; i < dropdownbox.length; i++){"
			+ "values = values + '"
			+ DELIMITER
			+ "' + dropdownbox.options[i].%s;" + "}" + "values;";
	final private static String DROPDOWN_OPTION_VALUE_PROPERTY = "value";
	final private static String DROPDOWN_OPTION_LABEL_PROPERTY = "text";
	final private static String ATTRIBUTE = "@";

	private Selenium selenium = null;

	private SeleniumFacade() {
		super();
	}

	public SeleniumFacade(final CommandProcessor processor) {
		this();
		selenium = new DefaultSelenium(processor);
	}

	public void start() {
		// TODO Wie kann selenium mit einem bereits ge�ffneten Browser
		// verbunden werden?
		selenium.start();
	}

	public void setTimeout(final String timeout) {
		selenium.setTimeout(timeout);
	}

	public void setSpeed(final String speed) {
		selenium.setSpeed(speed);
	}

	public void setBrowserLogLevel(final String browserLogLevel) {
		selenium.setBrowserLogLevel(browserLogLevel);
	}

	public void selectWindow(final String windowId) {
		selenium.selectWindow(windowId);
	}

	public void close() {
		selenium.close();
	}

	public void stop() {
		selenium.stop();
	}

	public void waitForPageToLoad(final String timeout) {
		selenium.waitForPageToLoad(timeout);
	}
	
	public void waitForCondition(final String script, final String timeout) {
		selenium.waitForCondition(script, timeout);
	}

	public void open(final String url) {
		selenium.open(url);
	}

	public void refresh() {
		selenium.refresh();
	}

	private String getLocator(final String prefix, final Locator locator,
			final String suffix) {
		return prefix + locator + suffix;
	}

	private String getLocator(final String prefix, final Locator locator) {
		return getLocator(prefix, locator, "");
	}

	private String getXPathLocator(final ElementLocator locator) {
		return getLocator("xpath=", locator);
	}

	private String getCSSLocator(final ElementLocator locator) {
		return getLocator("css=", locator);
	}

	/**
	 * Concatenates a prefix given by the configuration parameter
	 * Interactions.IDENTIFICATION_PREFIX and the parameter id. Afterwards it
	 * returns the concatenated value.
	 * 
	 * @param id
	 *            of the element for which the selenium locator shall be
	 *            generated
	 * @return a concatenation of a search prefix and the id
	 */
	private String getIdLocator(final ElementLocator locator) {
		// For some informations about partial id matching see following links:
		// http://clearspace.openqa.org/thread/6014 (problem description)
		// http://www.w3.org/TR/2001/CR-css3-selectors-20011113/#id-selectors
		// http://www.w3.org/TR/2001/CR-css3-selectors-20011113/#attribute-substrings
		// As follows an example css selector which matches all elements which
		// have a id with a ending "idvalueending"
		// css=[id$="idvalueending"]
		return getLocator(SeleniumWrapperConfiguration.IDENTIFICATION_PREFIX,
				locator, SeleniumWrapperConfiguration.IDENTIFICATION_SUFFIX);
	}

	private String getDOMLocator(final ElementLocator locator) {
		return getLocator("dom=", locator);
	}

	private String getSeleniumLocator(ElementLocator locator)
			throws LocatorNotSupportedException {
		if (locator instanceof ElementLocators) {
			locator = ((ElementLocators) locator).getPointer();
		}
		if (locator instanceof XPathLocator) {
			return getXPathLocator(locator);
		} else if (locator instanceof IdLocator) {
			return getIdLocator(locator);
		} else if (locator instanceof CSSLocator) {
			return getCSSLocator(locator);
		} else if (locator instanceof DOMLocator) {
			return getDOMLocator(locator);
		} else {
			throw new LocatorNotSupportedException(locator);
		}
	}

	private String getLinkLocator(final ElementLocator locator) {
		return getLocator("link=", locator);
	}

	private String getSeleniumLocatorForClick(final ElementLocator locator)
			throws LocatorNotSupportedException {
		String linkLocator = "";
		if (locator instanceof LabelLocator) {
			linkLocator = getLinkLocator(locator);
		} else {
			linkLocator = getSeleniumLocator(locator);
		}
		return linkLocator;
	}

	public void highlight(final ElementLocator locator)
			throws LocatorNotSupportedException {
		selenium.highlight(getLocatorOfAllPossible(locator));
	}

	private String getLocatorOfAllPossible(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return getSeleniumLocatorForClick(locator);
	}

	public boolean isElementPresent(final ElementLocator elementLocator)
			throws LocatorNotSupportedException {
		boolean elementPresent = false;
		if (elementLocator instanceof ElementLocators) {
			elementPresent = tryLocators(elementLocator);
		} else {
			elementPresent = selenium
					.isElementPresent(getLocatorOfAllPossible(elementLocator));
		}
		return elementPresent;
	}

	private boolean tryLocators(final ElementLocator elementLocator)
			throws LocatorNotSupportedException {
		boolean elementPresent = false;
		final ElementLocators locators = (ElementLocators) elementLocator;
		for (final ElementLocator locator : locators) {
			elementPresent = selenium
					.isElementPresent(getLocatorOfAllPossible(locator));
			if (elementPresent) {
				locators.setPointer(locator);
				break;
			}
		}
		return elementPresent;
	}

	public void click(ElementLocator locator)
			throws LocatorNotSupportedException {
		selenium.click(getSeleniumLocatorForClick(locator));
	}

	public Object isEditable(ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.isEditable(getSeleniumLocator(locator));
	}

	public void type(final ElementLocator locator, final String value)
			throws LocatorNotSupportedException {
		selenium.type(getSeleniumLocator(locator), value);
	}

	public void select(final ElementLocator locator,
			final OptionLocator optionLocator)
			throws LocatorNotSupportedException {
		selenium.select(getSeleniumLocator(locator),
				getSelectOptionLocator(optionLocator));
	}

	private String getSelectOptionLocator(final OptionLocator optionLocator)
			throws LocatorNotSupportedException {
		String locator = "";
		if (optionLocator instanceof com.hamburgsud.aludratest.locator.option.LabelLocator) {
			locator = getLabelOptionLocator(optionLocator);
		} else {
			throw new LocatorNotSupportedException(optionLocator);
		}
		return locator;
	}

	private String getLabelOptionLocator(final OptionLocator optionLocator) {
		return getLocator("label=", optionLocator);
	}

	public Object getText(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.getText(getSeleniumLocator(locator));
	}

	public Object isChecked(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.isChecked(getSeleniumLocator(locator));
	}

	public Object getSelectOptions(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.getSelectOptions(getSeleniumLocator(locator));
	}

	public Object getSelectedValue(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.getSelectedValue(getSeleniumLocator(locator));
	}

	public Object isVisible(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.isVisible(getSeleniumLocator(locator));
	}

	public Object getSelectedLabel(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.getSelectedLabel(getSeleniumLocator(locator));
	}

	public Object getValue(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return selenium.getValue(getSeleniumLocator(locator));
	}

	private String getWindowLocator(final WindowLocator windowLocator)
			throws LocatorNotSupportedException {
		if (windowLocator instanceof TitleLocator) {
			return getLocator("title=", windowLocator);
		} else {
			throw new LocatorNotSupportedException(windowLocator);
		}
	}

	public void selectWindow(final WindowLocator locator)
			throws LocatorNotSupportedException {
		selenium.selectWindow(getWindowLocator(locator));
	}

	public String[] getAllWindowTitles() {
		return selenium.getAllWindowTitles();
	}

	public String getTitle() {
		return selenium.getTitle();
	}

	public void windowMaximize() {
		selenium.windowMaximize();
	}

	public void windowFocus() {
		selenium.windowFocus();
	}

	public String getHtmlSource() {
		return selenium.getHtmlSource();
	}

	public String captureEntirePageScreenshotToString() {
		return selenium.captureEntirePageScreenshotToString("");
	}

	public String captureScreenshotToString() {
		return selenium.captureScreenshotToString();
	}

	public boolean hasFocus(final ElementLocator locator)
			throws LocatorNotSupportedException {
		final String seleniumLocator = getSeleniumLocator(locator);
		final String script = String.format(HAS_FOCUS_SCRIPT, seleniumLocator);
		final String result = selenium.getEval(script);
		final boolean returnValue = Boolean.parseBoolean(result);
		return returnValue;
	}

	public void focus(final ElementLocator locator)
			throws LocatorNotSupportedException {
		final String seleniumLocator = getSeleniumLocator(locator);
		selenium.focus(seleniumLocator);
	}

	private String[] getValues(final String csv) {
		final String[] values = csv.split(DELIMITER);
		final ArrayList<String> nonEmptyValues = new ArrayList<String>();
		for (String value : values) {
			if (value.length() > 0) {
				nonEmptyValues.add(value);
			}
		}
		return nonEmptyValues.toArray(new String[0]);
	}

	public String[] getDropDownValues(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return getPropertyValues(locator, DROPDOWN_OPTION_VALUE_PROPERTY);
	}

	private String[] getPropertyValues(final ElementLocator locator,
			final String propertyName) throws LocatorNotSupportedException {
		final String seleniumLocator = getSeleniumLocator(locator);
		final String script = String.format(DROPDOWN_SCRIPT, seleniumLocator,
				propertyName);
		final String csv = selenium.getEval(script);
		return getValues(csv);
	}

	public String[] getDropDownLabels(final ElementLocator locator)
			throws LocatorNotSupportedException {
		return getPropertyValues(locator, DROPDOWN_OPTION_LABEL_PROPERTY);
	}

	public String getAttributeValue(final ElementLocator locator,
			final String attributeName) throws LocatorNotSupportedException {
		final String seleniumLocator = getSeleniumLocator(locator);
		final String result = selenium.getAttribute(seleniumLocator + ATTRIBUTE
				+ attributeName);
		return result;
	}

	public void keyPress(final int keycode) {
		selenium.keyPressNative(String.valueOf(keycode));
	}

	public void doubleClick(final ElementLocator locator)
			throws LocatorNotSupportedException {
		final String seleniumLocator = getSeleniumLocator(locator);
		selenium.doubleClick(seleniumLocator);
	}

	public Object getTable(ElementLocator locator, int row, int col) throws LocatorNotSupportedException {
		String seleniumLocator = getSeleniumLocator(locator) + "." + row + "." + col;
		return selenium.getTable(seleniumLocator);
	}

	public Number getXPathCount(String xpath) {
		return selenium.getXpathCount(xpath);
	}

}
