/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.wrapper.selenium;

import java.util.ArrayList;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.TimeUnit;

import junit.framework.Assert;

import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.RootLogger;

import com.hamburgsud.aludratest.configuration.SeleniumWrapperConfiguration;
import com.hamburgsud.aludratest.exception.AludraTestException;
import com.hamburgsud.aludratest.exception.functional.ElementCouldNotBeFoundException;
import com.hamburgsud.aludratest.exception.functional.WindowCouldNotBeFoundException;
import com.hamburgsud.aludratest.exception.technical.LocatorNotSupportedException;
import com.hamburgsud.aludratest.exception.technical.TechnicalException;
import com.hamburgsud.aludratest.locator.element.ElementLocator;
import com.hamburgsud.aludratest.locator.option.OptionLocator;
import com.hamburgsud.aludratest.locator.window.WindowLocator;
import com.hamburgsud.aludratest.wrapper.Wrapper;
import com.hamburgsud.aludratest.wrapper.selenium.command.element.ElementCommand;
import com.hamburgsud.aludratest.wrapper.selenium.command.window.WindowCommand;
import com.hamburgsud.log4testing.core.data.attachment.Attachment;
import com.hamburgsud.log4testing.core.data.attachment.BinaryAttachment;
import com.hamburgsud.log4testing.core.data.attachment.StringAttachment;
import com.thoughtworks.selenium.CommandProcessor;
import com.thoughtworks.selenium.HttpCommandProcessor;
import com.thoughtworks.selenium.SeleniumException;

/**
 * 
 */
public class SeleniumWrapper implements Wrapper {

	public final static String WINDOW_LOCATOR_PREFIX = "title=";

	private static Queue<String> seleniumHosts = null;

	private static Logger logger;

	private SeleniumFacade selenium = null;

	private String usedSeleniumHost = null;

	static {
		logger = RootLogger.getLogger(SeleniumWrapper.class);
	}

	private static void initSeleniumHosts() {
		if (seleniumHosts == null) {
			seleniumHosts = new ConcurrentLinkedQueue<String>();
			seleniumHosts.addAll(SeleniumWrapperConfiguration.SELENIUM_HOSTS);
		}
	}

	private static String getSeleniumHost() {
		initSeleniumHosts();
		String seleniumHost = null;
		while (seleniumHost == null) {
			seleniumHost = seleniumHosts.poll();
			if (seleniumHost == null) {
				try {
					TimeUnit.MILLISECONDS.sleep(200);
				} catch (InterruptedException e) {
					logger.error("Thread interrupted while waiting for an "
							+ "available Selenium host.", e);
				}
			}
		}
		return seleniumHost;
	}

	public SeleniumWrapper() {
		init();
	}

	private void init() {
		initSelenium();
	}

	private void initSelenium() {
		usedSeleniumHost = getSeleniumHost();
		final CommandProcessor commandProcessor = new HttpCommandProcessor(
				usedSeleniumHost, SeleniumWrapperConfiguration.SELENIUM_PORT,
				SeleniumWrapperConfiguration.BROWSER,
				SeleniumWrapperConfiguration.URL_OF_AUT);
		selenium = new SeleniumFacade(commandProcessor);
		selenium.start();
		selenium.setTimeout(SeleniumWrapperConfiguration.TIMEOUT);
		selenium.setSpeed(SeleniumWrapperConfiguration.SPEED);
		selenium
				.setBrowserLogLevel(SeleniumWrapperConfiguration.BROWSER_LOG_LEVEL);
	}

	/**
	 * Closes the application under test respectively the main browser window
	 * and all child windows.
	 */
	private void closeApplicationUnderTest() {
		if (SeleniumWrapperConfiguration.CLOSE_TESTAPP_AFTER_EXECUTION) {
			selenium.selectWindow("null"); //$NON-NLS-1$
			selenium.close();
			selenium.stop();
		}
	}

	/**
	 * Closes the application under test.
	 */
	public void tearDown() {
		closeApplicationUnderTest();
		seleniumHosts.add(usedSeleniumHost);
	}

	public void waitForPageToLoad() {
		selenium.waitForPageToLoad(SeleniumWrapperConfiguration.TIMEOUT);
	}
	
	public void waitForEnabled(final ElementLocator locator) throws AludraTestException, InterruptedException {
		for (int second = 0;; second++) {
			if (second >= Integer.valueOf(SeleniumWrapperConfiguration.TIMEOUT).intValue()) {
				Assert.fail("timeout");
			}
			if (((Boolean) selenium.isEditable(locator))) {
				break;
			}
			Thread.sleep(1000);
		}
	}

	public void open(final String url) {
		selenium.open(url);
		waitForPageToLoad();
	}

	public void refresh() {
		selenium.refresh();
		waitForPageToLoad();
	}

	/**
	 * If highlighting is activated all HTML elements which are a target of a
	 * action get highlighted (marked yellow).
	 * 
	 * @param locator
	 *            of the element to highlight
	 * @throws LocatorNotSupportedException
	 */
	private void highlight(final ElementLocator locator)
			throws LocatorNotSupportedException {
		if (SeleniumWrapperConfiguration.HIGHLIGHT_COMMANDS) {
			try {
				selenium.highlight(locator);
			} catch (SeleniumException e) {
				// It does not matter if highlighting works or not, why a
				// possibly thrown exception must be caught to avoid test
				// execution termination.
				logger.trace("Highlighting does not work.", e);
			}
		}
	}

	/**
	 * Retries to evaluate the condition. If evaluation is successful, the
	 * method will return regularly. If not, this method will retry it with a
	 * pause of {@value #PAUSE_BETWEEN_RETRIES}ms between retries until a
	 * timeout of {@value BaseTestCase#TIMEOUT}ms exceeds.
	 * 
	 * @param condition
	 *            which will be evaluated on each retry
	 * @return true, if the condition could be executed successfully; otherwise
	 *         false will be returned.
	 * @throws TechnicalException
	 *             when the current thread is interrupted while it is sleeping
	 * @throws NumberFormatException
	 * @throws LocatorNotSupportedException
	 */
	private boolean retryUntilTimeout(final Condition condition)
			throws TechnicalException, LocatorNotSupportedException,
			NumberFormatException {
		final int numRetries = Integer
				.parseInt(SeleniumWrapperConfiguration.TIMEOUT)
				/ Integer
						.parseInt(SeleniumWrapperConfiguration.PAUSE_BETWEEN_RETRIES);
		boolean successful = false;
		for (int i = 0; i < numRetries; i++) {
			if (condition.eval()) {
				i = numRetries;
				successful = true;
			} else {
				try {
					Thread
							.sleep(Integer
									.parseInt(SeleniumWrapperConfiguration.PAUSE_BETWEEN_RETRIES));
				} catch (final InterruptedException e) {
					throw new TechnicalException(e);
				}
			}
		}
		return successful;
	}

	private void waitForElement(final ElementLocator locator)
			throws ElementCouldNotBeFoundException,
			LocatorNotSupportedException, TechnicalException {
		boolean elementIsFound = retryUntilTimeout(new Condition() {
			public boolean eval() throws LocatorNotSupportedException {
				return selenium.isElementPresent(locator);
			}
		});
		if (!elementIsFound) {
			throw new ElementCouldNotBeFoundException(locator);
		}
	}

	private void doBeforeDelegate(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		waitForElement(locator);
		highlight(locator);
	}

	// TODO Check if the following method is needed or delete it
	private List<Object> callElementCommand(final ElementCommand command,
			final ElementLocator... locators)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final List<Object> returnValues = new ArrayList<Object>();
		for (final ElementLocator elementLocator : locators) {
			doBeforeDelegate(elementLocator);
			Object returnValue = command.call(elementLocator);
			if (returnValue != null) {
				returnValues.add(returnValue);
			}
		}
		return returnValues;
	}

	private Object callElementCommand(final ElementLocator locator,
			final ElementCommand command)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final List<Object> returnValues = callElementCommand(command, locator);
		if (returnValues.isEmpty()) {
			return null;
		} else {
			return returnValues.get(0);
		}
	}

	public void click(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		callElementCommand(locator, new ElementCommand() {
			public Object call(final ElementLocator locator)
					throws LocatorNotSupportedException {
				selenium.click(locator);
				return null;
			}
		});
	}

	public boolean isElementPresent(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final Boolean returnValue = (Boolean) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator) {
						// this method will only be called if the element is
						// present why this method can directly return true
						return true;
					}

				});
		return returnValue.booleanValue();
	}

	public boolean isEditable(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final Boolean returnValue = (Boolean) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.isEditable(locator);
					}

				});
		return returnValue.booleanValue();
	}

	public void select(final ElementLocator locator,
			final OptionLocator optionLocator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		callElementCommand(locator, new ElementCommand() {

			public Object call(final ElementLocator locator)
					throws LocatorNotSupportedException {
				selenium.select(locator, optionLocator);
				return null;
			}

		});
	}

	public void type(final ElementLocator locator, final String value)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		callElementCommand(locator, new ElementCommand() {

			public Object call(final ElementLocator locator)
					throws LocatorNotSupportedException {
				selenium.type(locator, value);
				return null;
			}

		});
	}

	public String getText(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final String text = (String) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.getText(locator);
					}
				});
		return text;
	}

	public boolean isChecked(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final Boolean returnValue = (Boolean) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.isChecked(locator);
					}
				});
		return returnValue.booleanValue();
	}

	public String[] getSelectOptions(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final String[] selectedOptions = (String[]) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.getSelectOptions(locator);
					}
				});
		return selectedOptions;
	}

	public String getSelectedValue(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final String selectedValue = (String) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.getSelectedValue(locator);
					}
				});
		return selectedValue;
	}

	public boolean isVisible(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		Boolean returnValue = (Boolean) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.isVisible(locator);
					}
				});
		return returnValue.booleanValue();
	}

	public String getSelectedLabel(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final String selectedLabel = (String) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.getSelectedLabel(locator);
					}
				});
		return selectedLabel;
	}

	public String getValue(final ElementLocator locator)
			throws ElementCouldNotBeFoundException, TechnicalException,
			LocatorNotSupportedException {
		final String value = (String) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.getValue(locator);
					}
				});
		return value;
	}

	private void waitForWindow(final WindowLocator windowLocator)
			throws TechnicalException, WindowCouldNotBeFoundException,
			LocatorNotSupportedException, NumberFormatException {
		boolean windowIsFound = retryUntilTimeout(new Condition() {
			public boolean eval() {
				final String windowTitle = windowLocator.toString();
				final String[] titles = getAllWindowTitles();
				for (final String title : titles) {
					if (title.equalsIgnoreCase(windowTitle)) {
						return true;
					}
				}
				return false;
			}
		});
		if (!windowIsFound) {
			throw new WindowCouldNotBeFoundException(windowLocator);
		}
	}

	// TODO Think about objects for window ids and element ids
	private void callWindowCommand(final WindowLocator locator,
			final WindowCommand command) throws TechnicalException,
			WindowCouldNotBeFoundException, LocatorNotSupportedException,
			NumberFormatException {
		waitForWindow(locator);
		command.call(locator);
	}

	public void selectWindow(final WindowLocator locator)
			throws TechnicalException, WindowCouldNotBeFoundException,
			LocatorNotSupportedException, NumberFormatException {
		callWindowCommand(locator, new WindowCommand() {

			public void call(final WindowLocator locator)
					throws LocatorNotSupportedException {
				selenium.selectWindow(locator);
			}

		});
	}

	/**
	 * @see Selenium#getAllWindowTitles()
	 */
	public String[] getAllWindowTitles() {
		return selenium.getAllWindowTitles();
	}

	/**
	 * @see Selenium#getTitle()
	 */
	public String getTitle() {
		return selenium.getTitle();
	}

	/**
	 * @see Selenium#windowMaximize()
	 */
	public void windowMaximize() {
		selenium.windowMaximize();
	}

	/**
	 * @see Selenium#windowFocus()
	 */
	public void windowFocus() {
		selenium.windowFocus();
	}

	public Attachment getPageSource() {
		final String title = getTitle();
		final String pageSource = selenium.getHtmlSource();
		final Attachment attachment = new StringAttachment(title, pageSource,
				SeleniumWrapperConfiguration.PAGE_SOURCE_ATTACHMENT_EXTENSION);
		return attachment;
	}

	private Attachment getScreenshotAttachment(final String base64Data) {
		final String title = getTitle();
		final Base64 base64 = new Base64();
		final byte[] decodedData = base64.decode(base64Data);
		final Attachment attachment = new BinaryAttachment(title, decodedData,
				SeleniumWrapperConfiguration.SCREENSHOT_ATTACHMENT_EXTENSION);
		return attachment;
	}

	public Attachment getScreenshotOfTheWholeScreen() {
		final String base64Data = selenium
				.captureEntirePageScreenshotToString();
		final Attachment attachment = getScreenshotAttachment(base64Data);
		return attachment;
	}

	public Attachment getScreenshotOfThePage() {
		final String base64Data = selenium.captureScreenshotToString();
		final Attachment attachment = getScreenshotAttachment(base64Data);
		return attachment;
	}

	public boolean hasFocus(final ElementLocator locator)
			throws AludraTestException {
		boolean returnValue = (Boolean) callElementCommand(locator,
				new ElementCommand() {

					public Object call(ElementLocator locators)
							throws LocatorNotSupportedException {
						return selenium.hasFocus(locator);
					}

				});
		return returnValue;
	}

	public String[] getLabels(final ElementLocator locator)
			throws AludraTestException {
		return selenium.getDropDownLabels(locator);
	}

	public String[] getValues(final ElementLocator locator)
			throws AludraTestException {
		String[] returnValue = (String[]) callElementCommand(locator,
				new ElementCommand() {

					public Object call(ElementLocator locators)
							throws LocatorNotSupportedException {
						return selenium.getDropDownValues(locator);
					}
				});
		return returnValue;
	}

	public void focus(final ElementLocator locator) throws AludraTestException {
		callElementCommand(locator, new ElementCommand() {

			public Object call(ElementLocator locators)
					throws LocatorNotSupportedException {
				selenium.focus(locator);
				return null;
			}
		});
	}

	public String getAttributeValue(final ElementLocator elementLocator,
			final String attributeName) throws AludraTestException {
		String returnValue = (String) callElementCommand(elementLocator,
				new ElementCommand() {

					public Object call(ElementLocator locators)
							throws LocatorNotSupportedException {
						return selenium.getAttributeValue(elementLocator,
								attributeName);
					}
				});
		return returnValue;
	}

	public void keyPress(final int keycode) throws AludraTestException {
		selenium.keyPress(keycode);
	}

	public void doubleClick(final ElementLocator locator)
			throws AludraTestException {
		callElementCommand(locator, new ElementCommand() {

			public Object call(ElementLocator locators)
					throws LocatorNotSupportedException {
				selenium.doubleClick(locator);
				return null;
			}
		});
	}

	public void close() throws AludraTestException {
		selenium.close();
	}


	public String getTable(ElementLocator locator, final int row, final int col)
			throws ElementCouldNotBeFoundException,
			LocatorNotSupportedException, TechnicalException {
		final String text = (String) callElementCommand(locator,
				new ElementCommand() {

					public Object call(final ElementLocator locator)
							throws LocatorNotSupportedException {
						return selenium.getTable(locator, row, col);
					}
				});
		return text;
	}

	public void waitForInvisible(ElementLocator locator)
			throws AludraTestException, InterruptedException {
		for (int second = 0;; second++) {
			if (second >= Integer.valueOf(SeleniumWrapperConfiguration.TIMEOUT).intValue()) {
				Assert.fail("timeout");
			}
			if (!((Boolean) selenium.isVisible(locator))) {
				break;
			}
			Thread.sleep(1000);
		}		
	}

	public void waitForCondition(String script) {
		selenium.waitForCondition(script, SeleniumWrapperConfiguration.TIMEOUT);
	}

	public Number getXPathCount(String xpath) {
		return selenium.getXPathCount(xpath);
	}

}